Package: IsoformSwitchAnalyzeR
Type: Package
Title: Identify, Annotate and Visualize Isoform Switches with
        Functional Consequences from both short- and long-read RNA-seq
        data
Version: 2.6.0
Authors@R: c(person(given="Kristoffer",family="Vitting-Seerup", role=c("cre","aut"),email="k.vitting.seerup@gmail.com",comment = c(ORCID = "0000-0002-6450-0608")), person(given="Jeroen",family="Gilis", role="ctb", email="jeroen.gilis@ugent.be",comment = c(ORCID = "0000-0001-8415-0943")))
Description: Analysis of alternative splicing and isoform switches with predicted functional consequences (e.g. gain/loss of protein domains etc.) from quantification of all types of RNASeq by tools such as Kallisto, Salmon, StringTie, Cufflinks/Cuffdiff etc.
URL: http://bioconductor.org/packages/IsoformSwitchAnalyzeR/
BugReports: https://github.com/kvittingseerup/IsoformSwitchAnalyzeR/issues
License: GPL (>= 2)
Depends: R (>= 4.2), limma, DEXSeq, satuRn (>= 1.7.0), sva, ggplot2 (>=
        3.3.5), pfamAnalyzeR
Imports: methods, BSgenome, plyr, reshape2, gridExtra, Biostrings (>=
        2.50.0), IRanges, GenomicRanges, RColorBrewer, rtracklayer,
        VennDiagram, DBI, grDevices, graphics, stats, utils,
        GenomeInfoDb, grid, tximport (>= 1.7.1), tximeta (>= 1.7.12),
        edgeR, futile.logger, stringr, dplyr, magrittr, readr, tibble,
        XVector, BiocGenerics, RCurl, Biobase, SummarizedExperiment,
        tidyr, S4Vectors, BiocParallel, pwalign
Suggests: knitr, BSgenome.Hsapiens.UCSC.hg19, rmarkdown
VignetteBuilder: knitr
Collate: classes.R methods.R import_data.R test_isoform_switches.R
        analyze_ORF.R analyze_external_sequence_analysis.R
        analyze_switch_consequences.R isoform_plots.R
        plot_all_iso_switch.R high_level_functions.R tools.R
        analyze_alternative_splicing.R
biocViews: GeneExpression, Transcription, AlternativeSplicing,
        DifferentialExpression, DifferentialSplicing, Visualization,
        StatisticalMethod, TranscriptomeVariant, BiomedicalInformatics,
        FunctionalGenomics, SystemsBiology, Transcriptomics, RNASeq,
        Annotation, FunctionalPrediction, GenePrediction, DataImport,
        MultipleComparison, BatchEffect, ImmunoOncology
RoxygenNote: 6.0.1
git_url: https://git.bioconductor.org/packages/IsoformSwitchAnalyzeR
git_branch: RELEASE_3_20
git_last_commit: f466631
git_last_commit_date: 2024-10-29
Repository: Bioconductor 3.20
Date/Publication: 2024-10-29
NeedsCompilation: yes
Packaged: 2024-10-30 01:20:40 UTC; biocbuild
Author: Kristoffer Vitting-Seerup [cre, aut]
    (<https://orcid.org/0000-0002-6450-0608>),
  Jeroen Gilis [ctb] (<https://orcid.org/0000-0001-8415-0943>)
Maintainer: Kristoffer Vitting-Seerup <k.vitting.seerup@gmail.com>
